# imported from https://gitlab.com/ddcci-driver-linux/ddcci-driver-linux

%global	module	ddcci
%global	modulet	ddcci-backlight
%global pname ddcci-driver-linux


Name:		dkms-%{pname}
Version:	0.4.4
Release:	%mkrel 1
Source0:	%{pname}-%{version}.tar.xz
Group:		System/Kernel and hardware
License:	GPLv2
URL:		https://gitlab.com/ddcci-driver-linux/ddcci-driver-linux
Summary:	A pair of Linux kernel drivers for DDC/CI monitors.
BuildRoot:	%{_tmppath}/%{name}-%{version}-buildroot

# integrated in source since ver-rel-0.4.2
#Patch0:		ddcci-fix-k5.15-and-bump-version.patch

# fix define is any 
####Patch0:		ddcci-fix-defineIS_ANY-0.4.2.patch
###Patch1:		ddcci-makefile-dkms-%{version}.patch
###Patch2:		ddcci-driver-linux-makefile.patch

# ##Patch0:			ddcci-0.4.3-supKupto6.4.patch

Requires:   gcc >= 9.2.0

Provides:	kmod(%{module}.ko) = %{version}
Requires(post):		dkms
Requires(preun):	dkms

Obsoletes:	dkms-%{pname} < %{version}-%{release}

%description
ddcci-driver-linux
A pair of Linux kernel drivers for DDC/CI monitors. DDC/CI is a control protocol
for monitor settings supported by most monitors since about 2005.
It is based on ACCESS.bus (an early USB predecessor).

ddcci (bus driver)
This driver detects DDC/CI devices on DDC I²C busses, identifies them and
creates corresponding devices. As this is a I²C driver it won't be autoloaded
and must be manually loaded, for example by putting a line with ddcci
in /etc/modules.

sysfs interface
Each detected DDC/CI device gets a directory in /sys/bus/ddcci/devices.
The main device on a bus is named ddcci[I²C bus number].
Internal dependent devices are named ddcci[I²C bus number]i[hex address]
External dependent devices are named ddcci[I²C bus number]e[hex address]
There the following files export information about the device:

capabilities
The full ACCESS.bus capabilities string. It contains the protocol, type and
model of the device, a list of all supported command codes, etc.
See the ACCESS.bus spec for more information.

idProt
ACCESS.bus protocol supported by the device. Usually "monitor".

idType
ACCESS.bus device subtype. Usually "LCD" or "CRT".

idModel
ACCESS.bus device model identifier.
Usually a shortened form of the device model name.

idVendor
ACCESS.bus device vendor identifier.
Empty if the Identification command is not supported.

idModule
ACCESS.bus device module identifier.
Empty if the Identification command is not supported.

idSerial
32 bit device number. A fixed serial number if it's positive, a temporary serial
number if negative and zero if the Identification command is not supported.

Character device interface
For each DDC/CI device a character device in /dev/bus/ddcci/[I²C bus number]/
is created. The main device on the bus is named display.
Internal dependent devices are named i[hex address]
External dependent devices are named e[hex address]
These character devices can be used to issue commands to a DDC/CI device more
easily than over i2c-dev devices.
They should be opened unbuffered and may be opened with O_EXCL if you want
exclusive access. To send a command just write the command byte and the
arguments with a single write() operation. The length byte and checksum are
automatically calculated.
To read a response use read() with a buffer big enough for the expected answer.
NOTE: The maximum length of a DDC/CI message is 127 bytes.
An Example (in Python):
with open('/dev/bus/ddcci/3/display', 'r+b', buffering=0) as f:
	# Read contrast
	f.write(bytes([0x01, 0x12]))
	response = f.read(8)
	print("Contrast:", response[6] * 256 + response[7], "/", response[4] * 256 +
																 response[5])
The following error codes are used:

EAGAIN: there was no response yet or (with O_NONBLOCK) the device was in use
by another thread
EBADMSG: there was a response but the checksum didn't match
EBUSY: the device is opened exclusively by another thread (on open())
EINVAL: message too big (on write())
EIO: generic I/O failure
EMSGSIZE: the buffer was too small (on read())
ENOMEM: not enough free memory to allocate buffers (on open())

Lower layers may pass error codes not in this list like ENXIO, so be prepared
for that.

ddcci-backlight (monitor backlight driver)
For each monitor that supports accessing the Backlight Level White or
the Luminance property, a backlight device of type "raw" named like
the corresponding ddcci device is created. You can find them
in /sys/class/backlight/.

Limitations
Dependent device (sub devices using DDC/CI directly wired to the monitor,
like Calibration devices, IR remotes, etc.) aren't automatically detected.
You can force detection of internal dependent devices by setting the
autoprobe_addrs module parameter of ddcci.
You can force detection of external dependent devices by
writing "ddcci-dependent [address]" into /sys/bus/i2c/i2c-?/new_device.
There is no direct synchronization if you manually change the luminance with the
buttons on your monitor, as this can only be realized through polling and some
monitors close their OSD every time a DDC/CI command is received.
Monitor hotplugging is not detected. You need to detach/reattach the I²C driver
or reload the module.

%prep
%setup -qn %{pname}-%{version}

%autopatch -p1

find -type f -exec chmod 0644 {} \;


%install
%__rm -rf %{buildroot}
#%__install -dm755 %{buildroot}/lib/firmware
%__install -dm755 %{buildroot}%{_usrsrc}/%{pname}-%{version}-%{release}
%__cp -a . %{buildroot}%{_usrsrc}/%{pname}-%{version}-%{release}
#%__mv -f %{buildroot}%{_usrsrc}/%{pname}-%{version}-%{release}/regdb/* %{buildroot}/lib/firmware/
#%__rm -f %{buildroot}%{_usrsrc}/%{pname}-%{version}-%{release}/{CONTRIBUTORS.md,LICENSE,README.md}
%__cat <<EOF > %{buildroot}%{_usrsrc}/%{pname}-%{version}-%{release}/dkms.conf
PACKAGE_VERSION="%{version}"
PACKAGE_NAME="%{pname}"
BUILT_MODULE_NAME[0]="ddcci"
BUILT_MODULE_NAME[1]="ddcci-backlight"
BUILT_MODULE_LOCATION[0]="ddcci/"
BUILT_MODULE_LOCATION[1]="ddcci-backlight/"
DEST_MODULE_LOCATION[0]="/kernel/3rdparty/%{pname}"
DEST_MODULE_LOCATION[1]="/kernel/3rdparty/%{pname}"
MAKE="export IGNORE_CC_MISMATCH=1; 'make' clean KSRC=\${kernel_source_dir} SKIP_STACK_VALIDATION=1 all"
CLEAN="make clean"
AUTOINSTALL="yes"
EOF

%post -n %{name}
# rmmod can fail
/sbin/rmmod %{modulet} >/dev/null 2>&1 ||:
/sbin/rmmod %{module} >/dev/null 2>&1 ||:
## also clean olders in /extra... there was only modules, not a directory
find ./lib/modules/*/extra -type f -name "ddcci*ko.xz" -exec rm {} \; >/dev/null 2>&1
/usr/sbin/dkms --rpm_safe_upgrade add -m %{pname} -v %{version}-%{release} &&
/usr/sbin/dkms --rpm_safe_upgrade build -m %{pname} -v %{version}-%{release} &&
/usr/sbin/dkms --rpm_safe_upgrade install -m %{pname} -v %{version}-%{release} || :
if [ "$1" = 1 ]; then
/sbin/modprobe %{modulet} >/dev/null 2>&1
echo "%{modulet}" >> /etc/modprobe.preload
fi

%preun -n %{name}
/usr/sbin/dkms --rpm_safe_upgrade remove -m %{pname} -v %{version}-%{release} --all || :
if [ "$1" = 0 ]; then
/sbin/rmmod %{modulet} >/dev/null 2>&1
sed -i '/%{modulet}/ d' /etc/modprobe.preload
fi
## remove module-destdir just if it empty after dkms remove,(user's only deinstall after testing)... ##
## so prevent we not have empty folders in kernel-tree for next mklivecd ##
if [ ! -f /lib/modules/*/kernel/3rdparty/%{pname}/* ]; then
rm -rf /lib/modules/*/kernel/3rdparty/%{pname} >/dev/null 2>&1
fi
## also clean olders in /extra... there was only modules, not a directory
find ./lib/modules/*/extra -type f -name "ddcci*ko.xz" -exec rm {} \; >/dev/null 2>&1


%clean
%__rm -rf %{buildroot}

%files
%defattr(-,root,root)
%doc README.md LICENSE 
%{_usrsrc}/%{pname}-%{version}-%{release}
#/lib/firmware/*


%changelog
* Sat Feb 10 2024 onkelho <oh.quait at gmail.com> - 0.4.4-1quait2024
- new version rel
- - Support for Linux 6.2/6.3/6.4 added
- Replaced most semaphores with mutexes as per Linux kernel best practices

* Sat May 13 2023 onkelho <oh.quait at gmail.com> - 0.4.3-2pclos2022
- update
- add upstream patches for k-6.3...

* Wed Dec 28 2022 onkelho <oh.quait at gmail.com> - 0.4.3-1pclos2022
- fix build on kernel-6.1...

* Thu Jun 16 2022 onkelho <oh.quait at gmail.com> - 0.4.2-11pclos2022
- rebuild
- destination now /lib/modules/*/kernel/3rdparty/%{pname}

* Tue Jun 14 2022 onkelho <oh.quait at gmail.com> - 0.4.2-10pclos2022
- rebuild dkms.conf and deb's

* Sun Jun 12 2022 onkelho <oh.quait at gmail.com> - 0.4.2-9pclos2022
- rebuild - fixes in 3 makefiles and relevants
- modules now placed in /kernel/3rdparty/ddcci-driver-linux/....

* Sat Jun 11 2022 onkelho <oh.quait at gmail.com> - 0.4.2-7pclos2022
- rebuild - fix makefile and relevants

* Tue Jun 07 2022 onkelho <oh.quait at gmail.com> - 0.4.2-6pclos2022
- rebuild 

* Sat Apr 02 2022 onkelho <oh.quait at gmail.com> - 0.4.2-5pclos2022
- rebuild -- will fix execusion-error from rel 3-4  -removed
- fix in ddcci.c #define IS_ANY... 

* Mon Jan 31 2022 onkelho <oh.quait at gmail.com> - 0.4.2-2pclos2022
- updates kerv-5.17
- add export IGNORE_CC_MISMATCH=1;

* Sun Nov 21 2021 onkelho <oh.quait at gmail.com> - 0.4.2-1pclos2021
- new version per patch

* Fri Sep 03 2021 onkelho <oh.quait at gmail.com> - 0.4.1-1pclos2021
- new version update released

* Fri Jan 17 2020 onkelho <oh.quait at gmail.com> - 0.3.3-2pclos2020
- rebuild
- -Try bytewise writing if blockwise writing failed with EIO

* Tue Sep 24 2019 onkelho <oh.quait at gmail.com> - 0.3.3-1pclos2019
- new ver-rel
- dev to kernel-5.3

* Tue Aug 13 2019 tex - 0.3.2-3pclos2019
- rebuild against updates

* Tue Jun 25 2019 onkelho <oh.quait at gmail.com> - 0.3.2-2pclos2019
- rebuild new ver acpiligth

* Fri Jan 25 2019 onkelho <oh.quait@gmail.com> 0.3.2-1pclos2019
- initial build

